{% load pipeline %}
<html>
 <head>
  <meta http-equiv="Content-type" content="text/html; charset=utf-8">
  <title>Pipeline</title>
  {% stylesheet 'screen' %}
  {% javascript 'scripts' %}
 </head>
</html>
